import lib2to3.pgen2
from lib2to3.pygram import python_symbols as syms
from lib2to3.pytree import Leaf, Node
import os

from . import BaseSupportFix
from my2to3.trace import create_table
from my2to3.util import add_future, data2types


class FixDivisionSupport(BaseSupportFix):
  """Rewrites division_traced(..., a, b) into Py2/Py3-compatible division
  """
  basename = "division"

  @staticmethod
  def _analyze_data(data):
    types = data2types(data)

    if len(types) == 1:
      dividend, divisor = types[0]
      if dividend is divisor is int:
        return True, "automatic"

    return False, "manual"

  # Inspired by https://github.com/python/cpython/blob/e42b705188271da108de42b55d9344642170aa2b/Lib/lib2to3/fixes/fix_xrange.py#L14
  PATTERN = """
            power<'division_traced' trailer< '(' args=any ')' >
            rest=any* >
            """

  def transform(self, node, results):
    args = results['args']
    should_change = self.analyze_data(*[l.value for l in args.children[:-4:2]])
    if should_change:
      add_future(node, 'division')
      operator = Leaf(lib2to3.pgen2.token.DOUBLESLASH, "//")
    else:
      operator = Leaf(lib2to3.pgen2.token.SLASH, "/")
    operator.prefix = args.children[3].prefix

    node.replace(
      Node(
        syms.term,
        [
          args.children[2].clone(),
          operator,
          args.children[4].clone()
        ],
        prefix=node.prefix))
