# https://lab.nexedi.com/nexedi/erp5/snippets/475
import lib2to3.fixer_util
import lib2to3.pgen2
from lib2to3.pygram import python_symbols as syms
from lib2to3.pytree import Node

from . import BaseDynamicTraceFix
from my2to3.util import parse_type

# TODO: /=

class FixDivisionTrace(BaseDynamicTraceFix):
  """Rewrites a / b into division_traced(..., a, b)
  """
  basename = "division"
  traced_information = "dividend_type", "divisor_type"

  @staticmethod
  def _dynamic_trace(dividend, divisor):
    return (
      dividend / divisor,
      (parse_type(type(dividend)), parse_type(type(divisor)))
    )

  def match(self, node):
    if node.type == syms.term:
      # List the position of '/' sign nodes
      return [i for i, child in enumerate(node.children)
              if child.type == lib2to3.pgen2.token.SLASH]

  def transform(self, node, results):
    # Transforming "a / b" into "division_traced(id, a, b) is fairly trivial.
    # However, this is not the case when there are operations with the same
    # priority as the division (i.e. division, multiplication) which are
    # "in line", e.g.
    # - "a / b / c"
    # - "a * b / c * d / e"
    # - ...
    # That's why the logic is more complex here.

    previous_node = None

    for operator_pos in range(1, len(node.children), 2):

      # Get the two operators and the operation sign nodes
      children = [leaf.clone() for leaf in node.children[operator_pos - 1:operator_pos + 2]]
      if previous_node:
        # The first child is actually the previous operation
        children[0] = previous_node.clone()

      # Check if the current operation is a division or not
      if operator_pos in results:
        # It's a division operation

        # Replace the '/' sign by a comma
        comma = lib2to3.fixer_util.Comma()
        comma.prefix = children[1].prefix
        children[1] = comma

        previous_node = self.traced_call(node, children)
      else:
        # It's not a division operation
        previous_node = Node(syms.term, children)

    node.replace(previous_node)
