from lib2to3.pygram import python_symbols as syms

from . import BaseStaticTraceFix


class FixNestedExceptTrace(BaseStaticTraceFix):
  """This fixer detects scope bugs which can occur due to nested except clauses
  which use the same variable name.

  Try the following two snippets yourself in Python 2 and Python 3...

try:
  raise Exception("foo")
except Exception as e:
  try:
    raise Exception("bar")
  except Exception as e:
    print(e)
  print(e)

  ---

try:
  raise Exception("foo")
except Exception as e:
  try:
    raise Exception("bar")
  except Exception as e:
    print(e)
  raise

TODO: this does not cover all cases
  """
  basename = "nested_except"
  traced_information = "lineno_parent", "lineno_child"

  # https://github.com/python/cpython/blob/3549ca313a6103a3adb281ef3a849298b7d7f72c/Lib/lib2to3/fixes/fix_except.py#L39-L45
  PATTERN = """
  try_stmt< 'try' ':' (simple_stmt | suite)
                cleanup=(except_clause ':' (simple_stmt | suite))+
                tail=(['except' ':' (simple_stmt | suite)]
                      ['else' ':' (simple_stmt | suite)]
                      ['finally' ':' (simple_stmt | suite)]) >
  """

  def list_except_clause_children(self, node):
    for child in node.children:
      if child.type == syms.except_clause:
        yield child
      # BBB: yield from
      for c in self.list_except_clause_children(child):
        yield c

  def transform(self, node, results):
    try:
      except_clause, _, body = results['cleanup']
    except ValueError:
      return

    for child in self.list_except_clause_children(body):
      try:
        if except_clause.children[3] == child.children[3]:
          self.insert_trace(self.filename, None, None, except_clause.get_lineno(), child.get_lineno())
      except Exception:
        pass
