from collections import defaultdict
import lib2to3.fixer_util
import lib2to3.pgen2
from lib2to3.pygram import python_symbols as syms
from lib2to3.pytree import Node

from . import BaseDynamicTraceFix
from my2to3.util import parse_type


class FixRoundTrace(BaseDynamicTraceFix):
  """Rewrites round(a[, b]) into round_traced(..., a[, b])
  """
  basename = "round"
  traced_information = "number", "ndigits"

  @staticmethod
  def _dynamic_trace(number, ndigits=0):
    return (
      round(number, ndigits),
      (parse_type(type(number)), None if ndigits == 0 else parse_type(type(ndigits)))
    )

  # Inspired by https://github.com/python/cpython/blob/e42b705188271da108de42b55d9344642170aa2b/Lib/lib2to3/fixes/fix_xrange.py#L14
  PATTERN = """
            power<'round' trailer< '(' args=any ')' >
            rest=any* >
            """

  def transform(self, node, results):
    args = results['args']
    # e.g. round(1.23, 4)
    if args.type == syms.arglist:
      children = [leaf.clone() for leaf in args.children]
    # e.g. round(1.23)
    else:
      children = [
        args.clone()
      ]

    node.replace(self.traced_call(node, children))
